package com.hellokoding.jpa;

import com.hellokoding.jpa.model.Book;
import com.hellokoding.jpa.model.Publisher;
import com.hellokoding.jpa.repository.BookRepository;
import com.hellokoding.jpa.repository.PublisherRepository;
import com.vaadin.annotations.Theme;
import com.vaadin.server.VaadinRequest;
import com.vaadin.spring.annotation.SpringUI;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.UI;
import com.vaadin.ui.themes.ValoTheme;
import org.apache.commons.collections.CollectionUtils;
import org.hibernate.Hibernate;

import javax.annotation.PostConstruct;
import javax.transaction.Transactional;
import java.util.List;

@Theme(ValoTheme.THEME_NAME)
@SpringUI
public class HelloJpaApplicationUI extends UI {


    BookRepository bookRepository;

    PublisherRepository publisherRepository;

    ComboBox bookComboBox = new ComboBox();
    ComboBox newPublisherComboBox = new ComboBox();
    ComboBox oldPublisherComboBox = new ComboBox();

    public HelloJpaApplicationUI(BookRepository bookRepository, PublisherRepository publisherRepository) {
        this.bookRepository = bookRepository;
        this.publisherRepository = publisherRepository;

    }

    @Override
    protected void init(VaadinRequest vaadinRequest) {

        HorizontalLayout layout = new HorizontalLayout();
        layout.setSpacing(true);
        layout.setMargin(true);


        bookComboBox.addItems(bookRepository.findAll());

        bookComboBox.addValueChangeListener(valueChangeEvent -> {
            updateComboxes();

        });

        bookComboBox.setNewItemsAllowed(false);
        bookComboBox.setTextInputAllowed(false);

        newPublisherComboBox.setNewItemsAllowed(false);
        newPublisherComboBox.setTextInputAllowed(false);

        oldPublisherComboBox.setNewItemsAllowed(false);
        oldPublisherComboBox.setTextInputAllowed(false);

        Button addButton = new Button("Add");
        addButton.setStyleName("primary");
        addButton.addClickListener(clickEvent -> {
            Book book = (Book) bookComboBox.getValue();
            if(book != null){
                Publisher publisher = (Publisher) newPublisherComboBox.getValue();
                if(publisher != null){
                    book.addPublisher(publisher);
                    publisher.addBook(book);
                    bookRepository.save(book);
                    updateComboxes();
                }
            }
        });

        Button removeButton = new Button("Remove");
        removeButton.setStyleName("danger");
        removeButton.addClickListener(clickEvent -> {
            Book book = (Book) bookComboBox.getValue();
            if(book != null){
                Publisher publisher = (Publisher) oldPublisherComboBox.getValue();
                if(publisher != null){
                    book.removePublisher(publisher);
                    publisher.removeBook(book);
                    bookRepository.save(book);
                    updateComboxes();
                }
            }
        });


        layout.addComponents(bookComboBox, newPublisherComboBox, addButton, oldPublisherComboBox, removeButton);


        setContent(layout);
    }

    private void updateComboxes() {
        oldPublisherComboBox.removeAllItems();
        newPublisherComboBox.removeAllItems();
        Book book = (Book) bookComboBox.getValue();
        if(book != null) {

            List<Publisher> publisherList = publisherRepository.findAll();

            List<Publisher> oldPublisherList = book.getPublishers();
            oldPublisherComboBox.addItems(oldPublisherList);
            List<Publisher> newPubliserList = (List) CollectionUtils.disjunction(publisherList, oldPublisherList);
            newPublisherComboBox.addItems(newPubliserList);
        }
    }
}
